'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deepMapParents = exports.deepMapLeaves = exports.deepMap = exports.deepSetChildren = exports.deepSet = exports.deepGetChildren = exports.deepGet = exports.isLeaf = exports.keysToPathChildren = exports.keysToPath = exports.asList = undefined;

var _immutable = require('immutable');

//
// utils
//

var asList = exports.asList = function asList(input) {
  return _immutable.List.isList(input) ? input : (0, _immutable.List)(input);
};

var keysToPath = exports.keysToPath = function keysToPath(keys, childPath) {
  childPath = asList(childPath).toJS();
  return keys.reduce(function (fullPath, key) {
    return fullPath.concat(childPath).push(key);
  }, (0, _immutable.List)());
};

var keysToPathChildren = exports.keysToPathChildren = function keysToPathChildren(keys, childPath) {
  return keysToPath(keys, childPath).concat(childPath);
};

var isLeaf = exports.isLeaf = function isLeaf(item) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

  if (!_immutable.Iterable.isIterable(item)) {
    return true;
  }
  var children = item.getIn(childPath);
  return !children || children.isEmpty();
};

//
// get
//

var deepGet = exports.deepGet = function deepGet(keys, childPath) {
  var notSetValue = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  return function (input) {
    return input.getIn(keysToPath(keys, childPath), notSetValue);
  };
};

var deepGetChildren = exports.deepGetChildren = function deepGetChildren(keys, childPath) {
  var notSetValue = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];
  return function (input) {
    return input.getIn(keysToPathChildren(keys, childPath), notSetValue);
  };
};

//
// set
//

var deepSet = exports.deepSet = function deepSet(keys, value, childPath) {
  return function (input) {
    return input.setIn(keysToPath(keys, childPath), value);
  };
};

var deepSetChildren = exports.deepSetChildren = function deepSetChildren(keys, value, childPath) {
  return function (input) {
    return input.setIn(keysToPathChildren(keys, childPath), value);
  };
};

//
// map
//

var mapRecursive = function mapRecursive(self, childPath, mapper, options) {
  var keys = arguments.length <= 4 || arguments[4] === undefined ? (0, _immutable.List)() : arguments[4];

  var mapSelf = function mapSelf(self) {
    var children = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
    return mapper(self, keys, children);
  };

  if (isLeaf(self, childPath)) {
    return options.onlyParents ? self : mapSelf(self);
  }

  var mappedChildren = self.getIn(childPath).map(function (kid, key) {
    return mapRecursive(kid, childPath, mapper, options, keys.push(key));
  });

  var selfWithMappedChildren = childPath.isEmpty() ? self.merge(mappedChildren) : self.setIn(childPath, mappedChildren);

  return options.onlyLeaves ? selfWithMappedChildren : mapSelf(selfWithMappedChildren, mappedChildren);
};

var deepMap = exports.deepMap = function deepMap(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
  return function (input) {
    return mapRecursive(input, asList(childPath), mapper, {});
  };
};

var deepMapLeaves = exports.deepMapLeaves = function deepMapLeaves(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
  return function (input) {
    return mapRecursive(input, asList(childPath), mapper, { onlyLeaves: true });
  };
};

var deepMapParents = exports.deepMapParents = function deepMapParents(mapper) {
  var childPath = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
  return function (input) {
    return mapRecursive(input, asList(childPath), mapper, { onlyParents: true });
  };
};

//
// delete
//

// ...

//
// filter
//

// ...

//
// reduce
//

// ...

//
// sort
//

// ...

//
// deconstruct
//

// ...

//
// reconstruct
//

// ...