import {
  fromJS,
  Iterable,
  List,
  Map,
  OrderedMap
} from 'immutable';

//
// utils
//

export const asList = (input) => List.isList(input) ? input : List(input);

export const keysToPath = (keys, childPath) => {
  childPath = asList(childPath).toJS();
  return keys.reduce((fullPath, key) => {
    return fullPath.concat(childPath).push(key);
  }, List());
};

export const keysToPathChildren = (keys, childPath) => {
  return keysToPath(keys, childPath).concat(childPath);
};

export const isLeaf = (item, childPath = []) => {
  if(!Iterable.isIterable(item)) {
    return true;
  }
  const children = item.getIn(childPath);
  return !children || children.isEmpty();
};

//
// get
//

export const deepGet = (keys, childPath, notSetValue = null) => (input) => {
  return input.getIn(keysToPath(keys, childPath), notSetValue);
};

export const deepGetChildren = (keys, childPath, notSetValue = null) => (input) => {
  return input.getIn(keysToPathChildren(keys, childPath), notSetValue);
};

//
// set
//

export const deepSet = (keys, value, childPath) => (input) => {
  return input.setIn(keysToPath(keys, childPath), value);
};

export const deepSetChildren = (keys, value, childPath) => (input) => {
  return input.setIn(keysToPathChildren(keys, childPath), value);
};

//
// map
//

const mapRecursive = (self, childPath, mapper, options, keys = List()) => {
  const mapSelf = (self, children = null) => mapper(self, keys, children);
  
  if(isLeaf(self, childPath)) {
    return options.onlyParents ? self : mapSelf(self);
  }
  
  const mappedChildren = self
    .getIn(childPath)
    .map((kid, key) => mapRecursive(kid, childPath, mapper, options, keys.push(key)));
  
  const selfWithMappedChildren = childPath.isEmpty()
    ? self.merge(mappedChildren)
    : self.setIn(childPath, mappedChildren);
  
  return options.onlyLeaves
    ? selfWithMappedChildren
    : mapSelf(selfWithMappedChildren, mappedChildren);
};

export const deepMap = (mapper, childPath = []) => (input) => {
  return mapRecursive(input, asList(childPath), mapper, {});
};

export const deepMapLeaves = (mapper, childPath = []) => (input) => {
  return mapRecursive(input, asList(childPath), mapper, {onlyLeaves: true});
};

export const deepMapParents = (mapper, childPath = []) => (input) => {
  return mapRecursive(input, asList(childPath), mapper, {onlyParents: true});
};

//
// delete
//

// ...

//
// filter
//

// ...

//
// reduce
//

// ...

//
// sort
//

// ...

//
// deconstruct
//

// ...

//
// reconstruct
//

// ...

